import 'dart:io';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutter_ghost_content_api/flutter_ghost_content_api.dart';
import 'package:http/http.dart' as http;
import 'package:mockito/mockito.dart';

class MockClient extends Mock implements http.Client {}

Future<String> readFile(String path) async {
  File file = new File(path);
  return file.readAsString();
}

void main() {
  GhostContentAPIClient client;

  setUp(() {
    const url = 'https://demo.ghost.io';
    const version = 'v2';
    const key = '22444f78447824223cefc48062';
    client = GhostContentAPIClient(url: url, version: version, contentAPIKey: key);
  });



  test('fetches list of posts', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_posts.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts?key=' + client.contentAPIKey + '&limit=15&page=1';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PostsResponse postsResponse = await client.getPosts();

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postsResponse.posts.length, 15);
    expect(postsResponse.posts[0].authors, isNull);
    expect(postsResponse.posts[0].primaryAuthor, isNull);
    expect(postsResponse.posts[0].tags, isNull);
    expect(postsResponse.posts[0].primaryTag, isNull);
  });



  test('fetches list of posts with a filter', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_posts_filters_1.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts?key=' + client.contentAPIKey + '&limit=15&page=1&filter=tags:speeches';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    List<FilterExpression> filters = List<FilterExpression>();
    filters.add(FilterExpression(property: 'tags', value: 'speeches'));

    PostsResponse postsResponse = await client.getPosts(filters: filters);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postsResponse.posts.length, 2);
  });



  test('fetches list of posts with multiple filters', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_posts_filters_2.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts?key=' + client.contentAPIKey + '&limit=15&page=1&filter=tags:speeches&filter=featured:true';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    List<FilterExpression> filters = List<FilterExpression>();
    filters.add(FilterExpression(property: 'tags', value: 'speeches'));
    filters.add(FilterExpression(property: 'featured', value: 'true'));

    PostsResponse postsResponse = await client.getPosts(filters: filters);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postsResponse.posts.length, 3);
  });



  test('fetches list of posts including authors and tags', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_posts_include_authors_tags.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts?key=' + client.contentAPIKey + '&limit=15&page=1&include=authors,tags';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PostsResponse postsResponse = await client.getPosts(includes: [PostPageInclude.authors, PostPageInclude.tags]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postsResponse.posts.length, 15);
    expect(postsResponse.posts[0].authors, isNotNull);
    expect(postsResponse.posts[0].primaryAuthor, isNotNull);
    expect(postsResponse.posts[0].tags, isNotNull);
    expect(postsResponse.posts[0].primaryTag, isNotNull);
  });



  // todo test limit query param
  // todo test page query param




  test('fetches a single post by id', () async {
    MockClient mockClient = MockClient();

    String id = '5b7ada404f87d200b5b1f9c8';
    String mockResponse = await readFile('../test/resources/demo_content_post.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts/' + id + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PostResponse postResponse = await client.getPostById(id);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postResponse.posts.length, 1);
    expect(postResponse.posts[0].authors, isNull);
    expect(postResponse.posts[0].primaryAuthor, isNull);
    expect(postResponse.posts[0].tags, isNull);
    expect(postResponse.posts[0].primaryTag, isNull);
  });



  test('fetches a single post by id including authors and tags', () async {
    MockClient mockClient = MockClient();

    String id = '5b7ada404f87d200b5b1f9c8';
    String mockResponse = await readFile('../test/resources/demo_content_post_include_authors_tags.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts/' + id + '/?key=' + client.contentAPIKey + '&include=authors,tags';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PostResponse postResponse = await client.getPostById(id, includes: [PostPageInclude.authors, PostPageInclude.tags]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postResponse.posts.length, 1);
    expect(postResponse.posts[0].authors, isNotNull);
    expect(postResponse.posts[0].primaryAuthor, isNotNull);
    expect(postResponse.posts[0].tags, isNotNull);
    expect(postResponse.posts[0].primaryTag, isNotNull);
  });



  test('fetches a single post by slug', () async {
    MockClient mockClient = MockClient();

    String slug = 'welcome';
    String mockResponse = await readFile('../test/resources/demo_content_post.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts/slug/' + slug + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PostResponse postResponse = await client.getPostBySlug(slug);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postResponse.posts.length, 1);
    expect(postResponse.posts[0].authors, isNull);
    expect(postResponse.posts[0].primaryAuthor, isNull);
    expect(postResponse.posts[0].tags, isNull);
    expect(postResponse.posts[0].primaryTag, isNull);
  });



  test('fetches a single post by slug including authors and tags', () async {
    MockClient mockClient = MockClient();

    String slug = 'welcome';
    String mockResponse = await readFile('../test/resources/demo_content_post_include_authors_tags.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/posts/slug/' + slug + '/?key=' + client.contentAPIKey + '&include=authors,tags';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PostResponse postResponse = await client.getPostBySlug(slug, includes: [PostPageInclude.authors, PostPageInclude.tags]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(postResponse.posts.length, 1);
    expect(postResponse.posts[0].authors, isNotNull);
    expect(postResponse.posts[0].primaryAuthor, isNotNull);
    expect(postResponse.posts[0].tags, isNotNull);
    expect(postResponse.posts[0].primaryTag, isNotNull);
  });



  test('fetches list of authors', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_authors.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/authors?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    AuthorsResponse authorsResponse = await client.getAuthors();

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(authorsResponse.authors.length, 7);
    expect(authorsResponse.authors[0].count, isNull);
  });



  test('fetches list of authors including post count', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_authors_include_count_posts.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/authors?key=' + client.contentAPIKey + '&include=count.posts';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    AuthorsResponse authorsResponse = await client.getAuthors(includes: [AuthorInclude.countPosts]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(authorsResponse.authors.length, 7);
    expect(authorsResponse.authors[0].count, isNotNull);
    expect(authorsResponse.authors[0].count.posts, 1);
  });



  test('fetches a single author by id', () async {
    MockClient mockClient = MockClient();

    String id = '5c9a4da453c79000bf19a6f5';
    String mockResponse = await readFile('../test/resources/demo_content_author.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/authors/' + id + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    AuthorResponse authorResponse = await client.getAuthorById(id);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(authorResponse.authors.length, 1);
    expect(authorResponse.authors[0].count, isNull);
  });



  test('fetches a single author including post count', () async {
    MockClient mockClient = MockClient();

    String id = '5c9a4da453c79000bf19a6f5';
    String mockResponse = await readFile('../test/resources/demo_content_author_include_count_posts.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/authors/' + id + '/?key=' + client.contentAPIKey + '&include=count.posts';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    AuthorResponse authorResponse = await client.getAuthorById(id, includes: [AuthorInclude.countPosts]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(authorResponse.authors.length, 1);
    expect(authorResponse.authors[0].count, isNotNull);
    expect(authorResponse.authors[0].count.posts, 1);
  });




  test('fetches a single author by slug', () async {
    MockClient mockClient = MockClient();

    String slug = 'cameron';
    String mockResponse = await readFile('../test/resources/demo_content_author.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/authors/slug/' + slug + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    AuthorResponse authorResponse = await client.getAuthorBySlug(slug);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(authorResponse.authors.length, 1);
    expect(authorResponse.authors[0].count, isNull);
  });



  test('fetches a single author by slug including post count', () async {
    MockClient mockClient = MockClient();

    String slug = 'cameron';
    String mockResponse = await readFile('../test/resources/demo_content_author_include_count_posts.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/authors/slug/' + slug + '/?key=' + client.contentAPIKey + '&include=count.posts';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    AuthorResponse authorResponse = await client.getAuthorBySlug(slug, includes: [AuthorInclude.countPosts]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(authorResponse.authors.length, 1);
    expect(authorResponse.authors[0].count, isNotNull);
    expect(authorResponse.authors[0].count.posts, 1);
  });



  test('fetches list of tags', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_tags.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/tags?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    TagsResponse tagsResponse = await client.getTags();

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(tagsResponse.tags.length, 5);
    expect(tagsResponse.tags[0].count, isNull);
  });



  test('fetches list of tags including post count', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_tags_include_count_posts.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/tags?key=' + client.contentAPIKey + '&include=count.posts';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    TagsResponse tagsResponse = await client.getTags(includes: [TagInclude.countPosts]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(tagsResponse.tags.length, 5);
    expect(tagsResponse.tags[0].count, isNotNull);
    expect(tagsResponse.tags[0].count.posts, 1);
  });



  test('fetches a single tag by id', () async {
    MockClient mockClient = MockClient();

    String id = '5979a779df093500228e958a';
    String mockResponse = await readFile('../test/resources/demo_content_tag.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/tags/' + id + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    TagResponse tagResponse = await client.getTagById(id);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(tagResponse.tags.length, 1);
    expect(tagResponse.tags[0].count, isNull);
  });



  test('fetches a single tag including post count', () async {
    MockClient mockClient = MockClient();

    String id = '5979a779df093500228e958a';
    String mockResponse = await readFile('../test/resources/demo_content_tag_include_count_posts.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/tags/' + id + '/?key=' + client.contentAPIKey + '&include=count.posts';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    TagResponse tagResponse = await client.getTagById(id, includes: [TagInclude.countPosts]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(tagResponse.tags.length, 1);
    expect(tagResponse.tags[0].count, isNotNull);
    expect(tagResponse.tags[0].count.posts, 1);
  });




  test('fetches a single tag by slug', () async {
    MockClient mockClient = MockClient();

    String slug = 'fables';
    String mockResponse = await readFile('../test/resources/demo_content_tag.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/tags/slug/' + slug + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    TagResponse tagResponse = await client.getTagBySlug(slug);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(tagResponse.tags.length, 1);
    expect(tagResponse.tags[0].count, isNull);
  });



  test('fetches a single tag by slug including post count', () async {
    MockClient mockClient = MockClient();

    String slug = 'fables';
    String mockResponse = await readFile('../test/resources/demo_content_tag_include_count_posts.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/tags/slug/' + slug + '/?key=' + client.contentAPIKey + '&include=count.posts';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    TagResponse tagResponse = await client.getTagBySlug(slug, includes: [TagInclude.countPosts]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(tagResponse.tags.length, 1);
    expect(tagResponse.tags[0].count, isNotNull);
    expect(tagResponse.tags[0].count.posts, 1);
  });



  test('fetches list of pages', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_pages.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/pages?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PagesResponse pagesResponse = await client.getPages();

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(pagesResponse.pages.length, 1);
    expect(pagesResponse.pages[0].authors, isNull);
    expect(pagesResponse.pages[0].primaryAuthor, isNull);
    expect(pagesResponse.pages[0].tags, isNull);
    expect(pagesResponse.pages[0].primaryTag, isNull);
  });



  test('fetches list of pages including authors and tags', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_content_pages_include_authors_tags.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/pages?key=' + client.contentAPIKey + '&include=authors,tags';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PagesResponse pagesResponse = await client.getPages(includes: [PostPageInclude.authors, PostPageInclude.tags]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(pagesResponse.pages.length, 1);
    expect(pagesResponse.pages[0].authors, isNotNull);
    expect(pagesResponse.pages[0].primaryAuthor, isNotNull);
    expect(pagesResponse.pages[0].tags, isNotNull);
    expect(pagesResponse.pages[0].tags.length, 0);
    expect(pagesResponse.pages[0].primaryTag, isNull); // no primary tag as there are no tags at all
  });



  test('fetches a single page by id', () async {
    MockClient mockClient = MockClient();

    String id = '5979a4d6df093500228e9582';
    String mockResponse = await readFile('../test/resources/demo_content_page.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/pages/' + id + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PageResponse pageResponse = await client.getPageById(id);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(pageResponse.pages.length, 1);
    expect(pageResponse.pages[0].authors, isNull);
    expect(pageResponse.pages[0].primaryAuthor, isNull);
    expect(pageResponse.pages[0].tags, isNull);
    expect(pageResponse.pages[0].tags, isNull);
  });



  test('fetches a single page by id including authors and tags', () async {
    MockClient mockClient = MockClient();

    String id = '5979a4d6df093500228e9582';
    String mockResponse = await readFile('../test/resources/demo_content_page_include_authors_tags.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/pages/' + id + '/?key=' + client.contentAPIKey + '&include=authors,tags';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PageResponse pageResponse = await client.getPageById(id, includes: [PostPageInclude.authors, PostPageInclude.tags]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(pageResponse.pages.length, 1);
    expect(pageResponse.pages[0].authors, isNotNull);
    expect(pageResponse.pages[0].primaryAuthor, isNotNull);
    expect(pageResponse.pages[0].tags, isNotNull);
    expect(pageResponse.pages[0].tags.length, 0);
    expect(pageResponse.pages[0].primaryTag, isNull); // no primary tag as there are no tags at all
  });



  test('fetches a single page by slug', () async {
    MockClient mockClient = MockClient();

    String slug = 'about';
    String mockResponse = await readFile('../test/resources/demo_content_page.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/pages/slug/' + slug + '/?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PageResponse pageResponse = await client.getPageBySlug(slug);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(pageResponse.pages.length, 1);
    expect(pageResponse.pages[0].authors, isNull);
    expect(pageResponse.pages[0].primaryAuthor, isNull);
    expect(pageResponse.pages[0].tags, isNull);
    expect(pageResponse.pages[0].primaryTag, isNull);
  });



  test('fetches a single page by slug including authors and tags', () async {
    MockClient mockClient = MockClient();

    String slug = 'about';
    String mockResponse = await readFile('../test/resources/demo_content_page_include_authors_tags.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/pages/slug/' + slug + '/?key=' + client.contentAPIKey + '&include=authors,tags';

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    PageResponse pageResponse = await client.getPageBySlug(slug, includes: [PostPageInclude.authors, PostPageInclude.tags]);

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(pageResponse.pages.length, 1);
    expect(pageResponse.pages[0].authors, isNotNull);
    expect(pageResponse.pages[0].primaryAuthor, isNotNull);
    expect(pageResponse.pages[0].tags, isNotNull);
    expect(pageResponse.pages[0].tags.length, 0);
    expect(pageResponse.pages[0].primaryTag, isNull); // no primary tag as there are no tags at all
  });



  test('fetches settings', () async {
    MockClient mockClient = MockClient();

    String mockResponse = await readFile('../test/resources/demo_settings.json');
    String expectedEndpoint = client.url + '/ghost/api/' + client.version + '/content/settings?key=' + client.contentAPIKey;

    when(mockClient.get(expectedEndpoint)).thenAnswer((_) async => http.Response(mockResponse, 200, headers: {HttpHeaders.contentTypeHeader: 'application/json; charset=utf-8'}));

    client.client = mockClient;

    SettingsResponse settingsResponse = await client.getSettings();

    verify(mockClient.get(expectedEndpoint)).called(1);
    expect(settingsResponse.settings.title, 'Ghost');
  });
}
